aboutsummaryrefslogtreecommitdiff
path: root/src/runtime/slice.go
diff options
context:
space:
mode:
authorMartin Möhrmann <martisch@uos.de>2016-04-10 17:32:35 +0200
committerIan Lance Taylor <iant@golang.org>2016-04-10 23:39:46 +0000
commitad7448fe982d83de15deec9c55c56d0cd9261c6c (patch)
treee4d44cc145b562c0b1e184a4a5d2bec78b8cdfd3 /src/runtime/slice.go
parent5b3e5766bcc5e1090d0512a5916886ffc24ab246 (diff)
downloadgo-ad7448fe982d83de15deec9c55c56d0cd9261c6c.tar.gz
go-ad7448fe982d83de15deec9c55c56d0cd9261c6c.zip
runtime: speed up makeslice by avoiding divisions
Only compute the number of maximum allowed elements per slice once. name old time/op new time/op delta MakeSlice-2 55.5ns ± 1% 45.6ns ± 2% -17.88% (p=0.000 n=99+100) Change-Id: I951feffda5d11910a75e55d7e978d306d14da2c5 Reviewed-on: https://go-review.googlesource.com/21801 Run-TryBot: Ian Lance Taylor <iant@golang.org> TryBot-Result: Gobot Gobot <gobot@golang.org> Reviewed-by: Ian Lance Taylor <iant@golang.org>
Diffstat (limited to 'src/runtime/slice.go')
-rw-r--r--src/runtime/slice.go14
1 files changed, 11 insertions, 3 deletions
diff --git a/src/runtime/slice.go b/src/runtime/slice.go
index 4ab221056c..f36ec0b466 100644
--- a/src/runtime/slice.go
+++ b/src/runtime/slice.go
@@ -16,19 +16,27 @@ type slice struct {
// TODO: take uintptrs instead of int64s?
func makeslice(t *slicetype, len64, cap64 int64) slice {
// NOTE: The len > MaxMem/elemsize check here is not strictly necessary,
// but it produces a 'len out of range' error instead of a 'cap out of range' error
// when someone does make([]T, bignumber). 'cap out of range' is true too,
// but since the cap is only being supplied implicitly, saying len is clearer.
// See issue 4085.
+
+ maxElements := ^uintptr(0)
+ if t.elem.size > 0 {
+ maxElements = _MaxMem / t.elem.size
+ }
+
len := int(len64)
- if len64 < 0 || int64(len) != len64 || t.elem.size > 0 && uintptr(len) > _MaxMem/t.elem.size {
+ if len64 < 0 || int64(len) != len64 || uintptr(len) > maxElements {
panic(errorString("makeslice: len out of range"))
}
+
cap := int(cap64)
- if cap < len || int64(cap) != cap64 || t.elem.size > 0 && uintptr(cap) > _MaxMem/t.elem.size {
+ if cap < len || int64(cap) != cap64 || uintptr(cap) > maxElements {
panic(errorString("makeslice: cap out of range"))
}
+
p := newarray(t.elem, uintptr(cap))
return slice{p, len, cap}
}
@@ -16,19 +16,27 @@ type slice struct {
// TODO: take uintptrs instead of int64s?
func makeslice(t *slicetype, len64, cap64 int64) slice {
- // NOTE: The len > MaxMem/elemsize check here is not strictly necessary,
+ // NOTE: The len > maxElements check here is not strictly necessary,
// but it produces a 'len out of range' error instead of a 'cap out of range' error
// when someone does make([]T, bignumber). 'cap out of range' is true too,